//===========================================================================
//
// Crystalfontz Arduino Simple Demonstration Program
// for FTDI / BridgeTek EVE graphic accelerators.
// BT817
// Code written for Arduino MKR1000
// The MKR1000 has built-in WIFI
//
//---------------------------------------------------------------------------
//
// This is a simplified / refactored version of the code in FTDI's AN_275:
//
//  http://brtchip.com/wp-content/uploads/Support/Documentation/Application_Notes/ICs/EVE/AN_275_FT800_Example_with_Arduino.pdf
//
// I have added support for the BridgeTek BT817 EVE series.
//
// In the spirit of AN_275:
//
//   An “abstraction layer” concept was explicitly avoided in this
//   example. Rather, direct use of the Arduino libraries demonstrates
//   the simplicity of sending and receiving data through the FT800
//   while producing a graphic output.
//
// My main goal here is to be transparent about what is really happening
// from the high to lowest levels, without obfuscation, while still
// at least giving a nod to good programming practices.
//
// Plus, you probably don't have RAM and flash for all those fancy
// programming layers.
//
// The FTDI write offset (FWo) into the FT813's circular write write buffer
// is passed into and back from functions (FWol = FWo local) rather than being
// a global. Keeping track of the write offset avoids having to read that
// information from the FT813 before every SPI transaction.
//
// A nod to Rudolph R and company over at
//   https://www.mikrocontroller.net/topic/395608
//   https://github.com/RudolphRiedel/FT800-FT813
// for deep insight and lots of help in increasing our understanding
// of the fiddly bits of the EVE hardware and software architecture.
//
// 2022-02-28 Brent A. Crosby / Crystalfontz America, Inc.
// https://www.crystalfontz.com/products/eve-accelerated-tft-displays.php
//---------------------------------------------------------------------------
//This is free and unencumbered software released into the public domain.
//
//Anyone is free to copy, modify, publish, use, compile, sell, or
//distribute this software, either in source code form or as a compiled
//binary, for any purpose, commercial or non-commercial, and by any
//means.
//
//In jurisdictions that recognize copyright laws, the author or authors
//of this software dedicate any and all copyright interest in the
//software to the public domain. We make this dedication for the benefit
//of the public at large and to the detriment of our heirs and
//successors. We intend this dedication to be an overt act of
//relinquishment in perpetuity of all present and future rights to this
//software under copyright law.
//
//THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
//MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
//IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
//OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
//ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//OTHER DEALINGS IN THE SOFTWARE.
//
//For more information, please refer to <http://unlicense.org/>
//============================================================================
// Adapted from:
// FTDIChip AN_275 FT800 with Arduino - Version 1.0
//
// Copyright (c) Future Technology Devices International
//
// THIS SOFTWARE IS PROVIDED BY FUTURE TECHNOLOGY DEVICES INTERNATIONAL
// LIMITED "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL FUTURE TECHNOLOGY
// DEVICES INTERNATIONAL LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT,
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
// NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES LOSS OF USE,
// DATA, OR PROFITS OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
// OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// This code is provided as an example only and is not guaranteed by
// FTDI/BridgeTek. FTDI/BridgeTek accept no responsibility for any issues
// resulting from its use. By using this code, the developer of the final
// application incorporating any parts of this sample project agrees to take
// full responsible for ensuring its safe and correct operation and for any
// consequences resulting from its use.
//===========================================================================
#include <Arduino.h>
#include <SPI.h>
#include <stdarg.h>
#include <WiFi101.h>
#include <WiFiClient.h>
#include <ArduinoJson.h>
#include "arduino_secrets.h"
///////please enter your sensitive data in the Secret tab/arduino_secrets.h
char ssid[] = SECRET_SSID;    // your network SSID (name)
char pass[] = SECRET_PASS;    // your network password (use for WPA)
char key[] = SECRET_KEY;      // your API key
// Definitions for our circuit board and display.
#include "CFA10108_defines.h"

// Just the base of the URL you want to connect to
#define TEST_HOST "api.weatherlink.com"

int status = WL_IDLE_STATUS;
char server[] = TEST_HOST;    // name address for Google (using DNS)

// For HTTP requests
WiFiClient client;

//Static IP address configuration
IPAddress staticIP(192, 168, 1, 48); //ESP static ip
IPAddress gateway(192, 168, 1, 1);   //IP Address of your WiFi Router (Gateway)
IPAddress subnet(255, 255, 255, 0);  //Subnet mask
IPAddress dns(209,18,47,61);  //DNS

String dewpoint_f;
String observation_time;
String pressure_in;
String relative_humidity;
String temp_f;
String wind_dir;
String wind_mph;
String windchill_f;
String rain_day_in;
String rain_month_in;
String rain_year_in;
String pressure_tendency_string;
String sunrise;
String sunset;
String freedom_dewpoint_f;
String freedom_observation_time;
String freedom_temp_f;
String freedom_wind_mph;
String freedom_wind_dir;
String freedom_windchill_f;
String freedom_pressure_in;
String freedom_relative_humidity;
String freedom_rain_day_in;
String freedom_rain_month_in;
String freedom_rain_year_in;
String freedom_rain_storm_in;
String freedom_pressure_tendency_string;
String freedom_sunrise;
String freedom_sunset;
String freedom_ob_min;
int ob_min;            // extracted value of update minute
int mem_ob_min = 0;    // memory of last update minute
int pResistor = A1;    // Photoresistor at MKR1000 analog pin A1
int PR_Value;          // Store value from photoresistor (0-1023)
int BL_Val;            // Store value for display backlight (0-128)
int cf_Count = 0;      // keep track of connection fails
int upd_Count = 0;     // keep track of update faults
float freedom_rain_storm_flt; //rain storm float value

#if BUILD_SD
#include <SD.h>
#endif

// The very simple EVE library files
#include "EVE_base.h"
#include "EVE_draw.h"

// Our demonstrations of various EVE functions
#include "demos.h"

//===========================================================================
void setup()
{
//#if (DEBUG_LEVEL != DEBUG_NONE)
  // Initialize UART for debugging messages
  Serial.begin(115200);
  //while (!Serial) {
  //  ; // wait for serial port to connect. Needed for native USB port only
 // }
  
//#endif // (DEBUG_LEVEL != DEBUG_NONE)
  DBG_STAT("Begin\n");

  // check for the presence of the shield:
  if (WiFi.status() == WL_NO_SHIELD) {
    Serial.println("WiFi shield not present");
    // don't continue:
    while (true);
  }

  // attempt to connect to WiFi network:
  while (status != WL_CONNECTED) {
    Serial.print("Attempting to connect to SSID: ");
    Serial.println(ssid);
    // Connect to WPA/WPA2 network. Change this line if using open or WEP network:
    status = WiFi.begin(ssid, pass);

    // wait 10 seconds for connection:
    delay(10000);
  }
  Serial.println("Connected to wifi");

    Serial.println("\nStarting connection to server...");
  // if you get a connection, report back via serial:
  if (client.connect(server, 80)) {
    Serial.println("connected to server");
  } 

  //Initialize GPIO port states
  // Set CS# high to start - SPI inactive
  SET_EVE_CS_NOT;
  // Set PD# high to start
  SET_EVE_PD_NOT;
  SET_SD_CS_NOT;

  //Initialize port directions
  // EVE interrupt output (not used in this example)
  pinMode(EVE_INT, INPUT_PULLUP);
  // EVE Power Down (reset) input
  pinMode(EVE_PD_NOT, OUTPUT);
  // EVE SPI bus CS# input
  pinMode(EVE_CS_NOT, OUTPUT);
  // USD card CS
  pinMode(SD_CS, OUTPUT);
  // Optional pin used for LED or oscilloscope debugging.
  pinMode(DEBUG_LED, OUTPUT);
  // Set pResistor - A0 pin as an input
  pinMode(pResistor, INPUT);
  
  // Initialize SPI
  SPI.begin();
  //Bump the clock to 8MHz. Appears to be the maximum.
  SPI.beginTransaction(SPISettings(8000000, MSBFIRST, SPI_MODE0));
  DBG_GEEK("SPI initialzed to: 8MHz\n");

#if BUILD_SD
  // The prototype hardware appears to functon fine at 8MHz which
  // also appears to be the max that the ATmega328P can do.
  if (!SD.begin(8000000, SD_CS))
  {
    DBG_STAT("uSD card failed to initialize, or not present\n");
    //Reset the SPI clock to fast. SD card library does not clean up well.
    //Bump the clock to 8MHz. Appears to be the maximum.
    SPI.beginTransaction(SPISettings(8000000, MSBFIRST, SPI_MODE0));
  }
  else
  {
    DBG_STAT("uSD card initialized.\n");
  }
#endif

  //See if we can find the FTDI/BridgeTek EVE processor
  if (0 != EVE_Initialize())
  {
    DBG_STAT("Failed to initialize %s8%02X. Stopping.\n", EVE_DEVICE < 0x14 ? "FT" : "BT", EVE_DEVICE);
    while (1);
  }
  else
  {
    DBG_STAT("%s8%02X initialized.\n", EVE_DEVICE < 0x14 ? "FT" : "BT", EVE_DEVICE);
  }
} //  setup()
//===========================================================================
void loop()
{
  DBG_GEEK("Loop initialization.\n");

  //Get the current write pointer from the EVE
  uint16_t
  FWo;
  FWo = EVE_REG_Read_16(EVE_REG_CMD_WRITE);
  DBG_GEEK("Initial Offset Read: 0x%04X = %u\n", FWo , FWo);

  //Keep track of the RAM_G memory allocation
  uint32_t
  RAM_G_Unused_Start;
  RAM_G_Unused_Start = 0;
  DBG_GEEK("Initial RAM_G: 0x%08lX = %lu\n", RAM_G_Unused_Start, RAM_G_Unused_Start);

  // We need to keep track of the bitmap handles and where they are used.
  //
  // By default, bitmap handles 16 to 31 are used for built-in font and 15
  // is used as scratch bitmap handle by co-processor engine commands
  // CMD_GRADIENT, CMD_BUTTON and CMD_KEYS.
  //
  // For whatever reason, I am going to allocate handles from 14 to 0.
  uint8_t
  next_bitmap_handle_available;
  next_bitmap_handle_available = 14;
  DBG_GEEK("EVE_Initialize_Flash() . . . ");
  FWo = EVE_Initialize_Flash(FWo);
  DBG_GEEK("done.\n");
  uint8_t
  flash_status;
  flash_status = EVE_REG_Read_8(EVE_REG_FLASH_STATUS);
  DBG_GEEK_Decode_Flash_Status(flash_status);
#if (0 != PROGRAM_FLASH_FROM_USD)
  //Keep track of the current write pointer into flash.
  uint32_t
  Flash_Sector;
  Flash_Sector = 0;
  //Load the BLOB & write our image data to the flash from
  //the uSD card. This only needs to be executed once. It
  //uses RAM_G as scratch temporary memory, but does not
  //allocate any RAM_G.
  FWo = Initialize_Flash_From_uSD(FWo,
                                  RAM_G_Unused_Start,
                                  &Flash_Sector);
#else  // (0 != PROGRAM_FLASH_FROM_USD)
  DBG_GEEK("Not programming flash.\n");
#endif // (0 != PROGRAM_FLASH_FROM_USD)
#if (0 != BOUNCE_DEMO)
  DBG_STAT("Initialize_Bounce_Demo() . . .");
  Initialize_Bounce_Demo();
  DBG_STAT(" done.\n");
#endif // (0 != BOUNCE_DEMO)

#if (0 != LOGO_DEMO)
  DBG_STAT("Initialize_Logo_Demo() . . .");
  FWo = Initialize_Logo_Demo(FWo, &RAM_G_Unused_Start, next_bitmap_handle_available);
  //Keep track that we used a bitmap handle
  next_bitmap_handle_available--;
  DBG_STAT("  done.\n");
  DBG_GEEK("RAM_G after logo: 0x%08lX = %lu\n", RAM_G_Unused_Start, RAM_G_Unused_Start);
#endif // (0 != LOGO_DEMO)

#if (0 != BMP_DEMO)
  DBG_STAT("Initialize_Bitmap_Demo() . . .");
  FWo = Initialize_Bitmap_Demo(FWo, next_bitmap_handle_available);
  //Keep track that we used a bitmap handle
  next_bitmap_handle_available--;
  DBG_STAT("  done.\n");
  DBG_GEEK("RAM_G after bitmap: 0x%08lX = %lu\n", RAM_G_Unused_Start, RAM_G_Unused_Start);
#endif //(0 != BMP_DEMO)

#if (0 != SOUND_DEMO)
  DBG_STAT("Initialize_Sound_Demo() . . .");
  FWo = Initialize_Sound_Demo(FWo, &RAM_G_Unused_Start);
  DBG_STAT("  done.\n");
  DBG_GEEK("RAM_G after sound: 0x%08lX = %lu\n", RAM_G_Unused_Start, RAM_G_Unused_Start);
#endif // (0 != SOUND_DEMO)

#if (0 != MARBLE_DEMO)
  DBG_STAT("Initialize_Marble_Demo() . . .");
  FWo = Initialize_Marble_Demo(FWo, &RAM_G_Unused_Start, next_bitmap_handle_available);
  //Keep track that we used a bitmap handle
  next_bitmap_handle_available--;
  DBG_STAT("  done.\n");
  DBG_GEEK("RAM_G after marble: 0x%08lX = %lu\n", RAM_G_Unused_Start, RAM_G_Unused_Start);
#endif //MARBLE_DEMO

#if (0 != TOUCH_DEMO)
  //Bitmask of valid points in the array
  uint8_t
  points_touched_mask;
#if (EVE_TOUCH_TYPE == EVE_TOUCH_RESISTIVE)
  DBG_GEEK("Resistive touch, single point.\n");
  int16_t
  x_points[1];
  int16_t
  y_points[1];
#endif // (EVE_TOUCH_TYPE == EVE_TOUCH_RESISTIVE)

#if (EVE_TOUCH_TYPE == EVE_TOUCH_CAPACITIVE)
  DBG_GEEK("Capacitive touch, multiple points.\n");
  int16_t
  x_points[5];
  int16_t
  y_points[5];
#endif // (EVE_TOUCH_TYPE == EVE_TOUCH_CAPACITIVE)

#if (0 != EVE_TOUCH_CAL_NEEDED)
  DBG_STAT("Touch calibration . . .");
  //Ask the user to calibrate the touch screen.
  FWo = EVE_Busy_StopFF(FWo,
                        //clear color
                        EVE_ENC_CLEAR_COLOR_RGB(0x00, 0x00, 0xFF),
                        //text color
                        EVE_ENC_COLOR_RGB(0xFF, 0xFF, 0xFF),
                        "Calibrate Touch");
  FWo = Calibrate_Touch(FWo);
  DBG_STAT("done.\n");
#else  // (0 != EVE_TOUCH_CAL_NEEDED)
  DBG_GEEK("Display specifies no touch calibration is needed.\n");
#endif // (0 != EVE_TOUCH_CAL_NEEDED)

#if (EVE_TOUCH_TYPE==EVE_TOUCH_CAPACITIVE)
  //Capacitive.
  //Set multi-touch mode. You gots to do this _after_ the calibration.
  EVE_REG_Write_8(EVE_REG_CTOUCH_EXTENDED, EVE_CTOUCH_MODE_EXTENDED);

#if (EVE_TOUCH_CAP_DEVICE == EVE_CAP_DEV_FT5316)
  //For some reason, the combination of the BT817 and the
  //FT5316 leave the screen in a "touched" state at this
  //point. It does not matter if the calibration has
  //been called or if the default FT5316 cal is used.
  //Empirically determined, resetting the touch
  //engine seems to fix it.
  EVE_REG_Write_8(EVE_REG_CPURESET, 2); //touch stopped
  delay(1);
  EVE_REG_Write_8(EVE_REG_CPURESET, 0); //all running
  delay(1);
#endif // (EVE_TOUCH_CAP_DEVICE == EVE_CAP_DEV_FT5316)
#endif // (EVE_TOUCH_TYPE==EVE_TOUCH_CAPACITIVE)

  //Wait for the user to get their finger off the display--except
  //the cal does not exit until the finger is up -- so why does
  //this loop get executed up to ~ 85 times (for capacitive with
  //Goodix)? Removing this loop gives an initial, invalid, "false
  //touch" when the touch is read in the main loop. Very odd.
  //It does not hurt for any other controller.
  do
  {
    //Read the touch screen.
    points_touched_mask = Read_Touch(x_points, y_points);
  } while (0 != points_touched_mask);
#endif // (0 != TOUCH_DEMO)

#if (0 != VIDEO_DEMO)
  FWo = Initialize_Video_Demo(FWo,
                              &RAM_G_Unused_Start,
                              next_bitmap_handle_available);
  //Keep track that we used a bitmap handle
  next_bitmap_handle_available--;
  DBG_GEEK("RAM_G after video: 0x%08lX = %lu\n", RAM_G_Unused_Start, RAM_G_Unused_Start);
#endif // (0 != VIDEO_DEMO)

  DBG_STAT("Initialization complete, entering main loop.\n");

  while (1)
  {
    //========== FRAME SYNCHRONIZING ==========
    // Wait for graphics processor to complete executing the current command
    // list. This happens when EVE_REG_CMD_READ matches EVE_REG_CMD_WRITE, indicating
    // that all commands have been executed.  We have a local copy of
    // EVE_REG_CMD_WRITE in FWo.
    //
    // This appears to only occur on frame completion, which is nice since it
    // allows us to step the animation along at a reasonable rate.
    //
    // If possible, I have tweaked the timing on the Crystalfontz displays
    // to all have ~60Hz frame rate.
    FWo = Wait_for_EVE_Execution_Complete(FWo);
    //delay(17);

#if TOUCH_DEMO
    //Read the touch screen.
    points_touched_mask = Read_Touch(x_points, y_points);
#endif // TOUCH_DEMO

#if SOUND_DEMO
    //See if we should play a sound. The sound will synchronize
    //with the the start of the logo rotation.
    //If the previous sound is still playing it will wait until the
    //next time we call it.
    Start_Sound_Demo_Playing();
#endif //SOUND_DEMO

    //========== START THE DISPLAY LIST ==========
    // Start the display list
    FWo = EVE_Cmd_Dat_0(FWo, (EVE_ENC_CMD_DLSTART));

    // Set the default clear color to black
    FWo = EVE_Cmd_Dat_0(FWo, EVE_ENC_CLEAR_COLOR_RGB(0, 0, 0));

    // Clear the screen - this and the previous prevent artifacts between lists
    FWo = EVE_Cmd_Dat_0(FWo,
                        EVE_ENC_CLEAR(1, 1, 1)); //CLR_COL,CLR_STN,CLR_TAG
    //========== ADD GRAPHIC ITEMS TO THE DISPLAY LIST ==========
    //Fill background with white
    FWo = EVE_Filled_Rectangle(FWo,
                               0, 0, LCD_WIDTH - 1, LCD_HEIGHT - 1);
    FWo = EVE_Cmd_Dat_0(FWo,
                        EVE_ENC_COLOR_RGB(0x00, 0x00, 0x00)); //black font

#if (0 != BMP_DEMO)
    FWo = Add_Bitmap_To_Display_List(FWo);
#endif // BMP_DEMO

//++++++++++++++++++++++++++++++++++++++++++++++ HTTP Request +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  //delay(900000); // delay 15 minutes
  //delay(300000); // delay 5 minutes
  delay(60000); //delay 1 minute
  makeHTTPRequest();
  
//++++++++++++++ Create Char Arrays ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        int str_len = freedom_dewpoint_f.length() + 1;
        char dew_F[str_len];
        freedom_dewpoint_f.toCharArray(dew_F, str_len); 

        int str_len_0 = freedom_observation_time.length() + 1;
        char ob_Time[str_len_0];
        freedom_observation_time.toCharArray(ob_Time, str_len_0);

        int str_len_1 = freedom_temp_f.length() + 1;
        char temp_F[str_len_1];
        freedom_temp_f.toCharArray(temp_F, str_len_1);

        int str_len_2 = freedom_wind_mph.length() + 1;
        char wind_Mph[str_len_2];
        freedom_wind_mph.toCharArray(wind_Mph, str_len_2);

        int str_len_3 = freedom_wind_dir.length() + 1;
        char wind_Dir[str_len_3];
        freedom_wind_dir.toCharArray(wind_Dir, str_len_3);

        int str_len_4 = freedom_windchill_f.length() + 1;
        char wind_Chill[str_len_4];
        freedom_windchill_f.toCharArray(wind_Chill, str_len_4);

        int str_len_5 = freedom_pressure_in.length() + 1;
        char press_In[str_len_5];
        freedom_pressure_in.toCharArray(press_In, str_len_5);

        int str_len_6 = freedom_relative_humidity.length() + 1;
        char rel_Hum[str_len_6];
        freedom_relative_humidity.toCharArray(rel_Hum, str_len_5);

        int str_len_7 = freedom_rain_day_in.length() + 1;
        char rain_Day[str_len_7];
        freedom_rain_day_in.toCharArray(rain_Day, str_len_7);

        int str_len_8 = freedom_rain_month_in.length() + 1;
        char rain_Mo[str_len_8];
        freedom_rain_month_in.toCharArray(rain_Mo, str_len_8);

        int str_len_9 = freedom_rain_year_in.length() + 1;
        char rain_Yr[str_len_9];
        freedom_rain_year_in.toCharArray(rain_Yr, str_len_9);

        int str_len_10 = freedom_pressure_tendency_string.length() + 1;
        char press_Str[str_len_10];
        freedom_pressure_tendency_string.toCharArray(press_Str, str_len_10);
        
        int str_len_11 = freedom_sunrise.length() + 1;
        char sun_Rise[str_len_11];
        freedom_sunrise.toCharArray(sun_Rise, str_len_11);
 
        int str_len_12 = freedom_sunset.length() + 1;
        char sun_Set[str_len_12];
        freedom_sunset.toCharArray(sun_Set, str_len_12);

        String percent_symbol = "%";
        int str_len_14 = percent_symbol.length() + 1;
        char pct_Sym[str_len_14];
        percent_symbol.toCharArray(pct_Sym, str_len_14);
        //Serial.print("str_len_14 = ");
        //Serial.println(str_len_14);
                
        String degree_symbol = "⁰";
        int str_len_15 = degree_symbol.length() + 1;
        char deg_Sym[str_len_15];
        degree_symbol.toCharArray(deg_Sym, str_len_15);
        //Serial.print("str_len_15 = ");
        //Serial.println(str_len_15);

        int str_len_16 = freedom_rain_storm_in.length() + 1;
        char rain_Storm[str_len_16];
        freedom_rain_storm_in.toCharArray(rain_Storm, str_len_16);
  /*     
        int index;
        for(index=0;index<=3;index++)
        {
        DBG_STAT("pct_Sym[%2d]= %3d (dec) = 0x%02X (hex) = %c (character) \n",
        index, pct_Sym[index], pct_Sym[index],
        isPrintable(pct_Sym[index])?pct_Sym[index]:'?');
        }

        int index_1;
        for(index_1=0;index_1<=3;index_1++)
        {
        DBG_STAT("deg_Sym[%2d]= %3d (dec) = 0x%02X (hex) = %c (character) \n",
        index_1, deg_Sym[index_1], deg_Sym[index_1],
        isPrintable(deg_Sym[index_1])?deg_Sym[index_1]:'?');
        }
        
*/
        //int hexValue = 0xA6;
        //char character = (char)(hexValue & 0xFF);
        
        
        
        //DBG_STAT(" the value of character as a string is: \"%s\"\n",character); 


           
  //========== START THE DISPLAY LIST ======================================================================================================================

  PR_Value = analogRead(pResistor);
  Serial.print("PR_Value = ");
  Serial.println(int(PR_Value));

  BL_Val = map(analogRead(pResistor), 0, 1023, 1, 128);
  Serial.print("BL_Value = ");
  Serial.println(int(BL_Val));

  if(BL_Val < 1)
    {
    BL_Val=1;
    }
  if(128<BL_Val)
    {
    BL_Val=128;
    }

    //Write the actual value
    EVE_REG_Write_8(EVE_REG_PWM_DUTY,BL_Val);

          // Set font 16 to 31 size
       FWo=EVE_Cmd_Dat_2(FWo,
                           EVE_ENC_CMD_ROMFONT,16,31);  //32 to 34

            // Set font 20 to 34 size
        FWo=EVE_Cmd_Dat_2(FWo,
                           EVE_ENC_CMD_ROMFONT,20,34);  //32 to 34                  

        FWo=EVE_Cmd_Dat_0(FWo,
                       EVE_ENC_COLOR_RGB(0xFF,0xFF,0x00));//yellow font
                      
        FWo=EVE_PrintF(FWo,
                       95,  //X horizontal
                       160,  //Y vertical
                       30,         //Font
                       EVE_OPT_CENTER, //Options
                       //"%s",//sunrise
                       sun_Rise);
                      
        FWo=EVE_Cmd_Dat_0(FWo,
                       EVE_ENC_COLOR_RGB(0xFF,0x99,0x00));//orange font

        FWo=EVE_PrintF(FWo,
                       937,  //X
                       160,  //Y
                       30,         //Font
                       EVE_OPT_CENTER, //Options
                       "%s",//sunset
                       sun_Set);
                 
        FWo=EVE_Cmd_Dat_0(FWo,
                       EVE_ENC_COLOR_RGB(0x00,0xFF,0x00));//green

        FWo=EVE_PrintF(FWo,
                       310,  //X horizontal
                       158,  //Y vertical
                       16,         //Font
                       EVE_OPT_RIGHTX, //Options
                      "%s",//temperature
                       temp_F);
/*
        FWo=EVE_PrintF(FWo,
                       370,  //X horizontal
                       165,  //Y vertical
                       19,         //Font
                       EVE_OPT_CENTER, //Options
                      "\277");//temperature
                      "\F8*");//temperature 
*/                      
         FWo=EVE_PrintF(FWo,
                       517,  //X
                       158,  //Y
                       16,         //Font
                       EVE_OPT_RIGHTX, //Options
                       "%s",//windchill
                       wind_Chill);

         FWo=EVE_PrintF(FWo,
                       755,  //X horizontal
                       158,  //Y vertical
                       16,         //Font
                       EVE_OPT_RIGHTX, //Options
                       "%s",//dewpoint
                       dew_F);

         FWo=EVE_Cmd_Dat_0(FWo,
                       EVE_ENC_COLOR_RGB(0x88,0x00,0xFF));//violet

         FWo=EVE_PrintF(FWo,
                       512,  //X
                       265,  //Y
                       16,         //Font
                       EVE_OPT_CENTER, //Options
                       "%s @ %s MPH",//wind dir
                       wind_Dir,
                       wind_Mph);
                       
         FWo=EVE_Cmd_Dat_0(FWo,
                       EVE_ENC_COLOR_RGB(0x10,0x00,0xFF));//blue
                       
         FWo=EVE_PrintF(FWo,
                       200,  //X horizontal
                       380,  //Y vertical
                       16,         //Font
                       EVE_OPT_CENTER, //Options
                       "%s \"Hg",//pressure in.
                       press_In);

         FWo=EVE_PrintF(FWo,
                       512,  //X
                       365,  //Y
                       25,         //Font
                       EVE_OPT_CENTER, //Options
                       //"%s",//pressure tendency
                       press_Str);

         FWo=EVE_PrintF(FWo,
                       850,  //X horizontal
                       380,  //Y vertical
                       16,         //Font
                       EVE_OPT_CENTER, //Options
                       "%s %s",//humidity
                       rel_Hum,
                       pct_Sym);

         FWo=EVE_Cmd_Dat_0(FWo,
                       EVE_ENC_COLOR_RGB(0x42,0xF5,0xCE));//aqua         

         FWo=EVE_PrintF(FWo,
                       225,  //X horizontal
                       480,  //Y vertical
                       16,         //Font
                       EVE_OPT_CENTER, //Options
                       "%s \"",//rain day
                       rain_Day);
                                            
         FWo=EVE_PrintF(FWo,
                       545,  //X
                       480,  //Y
                       16,         //Font
                       EVE_OPT_CENTER, //Options
                       "%s \"",//rain month
                       rain_Mo);
                                             
         FWo=EVE_PrintF(FWo,
                       845,  //X horizontal
                       480,  //Y vertical
                       16,         //Font
                       EVE_OPT_CENTER, //Options
                       "%s \"",//rain year
                       rain_Yr);                       

        FWo=EVE_Cmd_Dat_0(FWo,
                       EVE_ENC_COLOR_RGB(0xff,0x00,0x00));//red

        FWo=EVE_PrintF(FWo,
                       512,  //X horizontal
                       552,  //Y vertical
                       24,         //Font
                       EVE_OPT_CENTER, //Options
                       //"%s",//observation time
                       ob_Time);

      if (ob_min == mem_ob_min) {

        upd_Count ++; //keep track of update fails
        Serial.print("Update failed count = ");
        Serial.println(upd_Count);
        
         FWo=EVE_PrintF(FWo,
                       10,  //X horizontal
                       590,  //Y vertical
                       21,         //Font
                       EVE_OPT_CENTER, //Options
                       "%d ",//update not new data
                       ob_min);       
            }
        else
            {
               upd_Count = 0;
               FWo=EVE_PrintF(FWo,
                       10,  //X horizontal
                       590,  //Y vertical
                       21,         //Font
                       EVE_OPT_CENTER, //Options
                       "OK");//update OK                     
              }

        //cf_Count = 5; //testing      

        if ((cf_Count >= 5)||(upd_Count >= 5)) {
          FWo=EVE_PrintF(FWo,
                       512,  //X horizontal
                       265,  //Y vertical
                       20,         //Font
                       EVE_OPT_CENTER, //Options
                       "CHECK PC !");//check PC 
        }
    
       FWo=EVE_Cmd_Dat_0(FWo,
                       EVE_ENC_COLOR_RGB(0xff,0xff0,0xff));//white

        if (freedom_rain_storm_flt != 0.00) {
          FWo=EVE_PrintF(FWo,
                       300,  //X horizontal
                       430,  //Y vertical
                       25,         //Font
                       EVE_OPT_CENTER, //Options
                       "Rain Storm");//check PC
                       
            FWo=EVE_PrintF(FWo,
                       680,  //X
                       430,  //Y
                       25,         //Font
                       EVE_OPT_CENTER, //Options
                       "%s \"",//rain storm
                       rain_Storm);         
               }

         mem_ob_min = ob_min; //store new observation minute
         Serial.print("mem_ob_min_2 = ");
         Serial.println(mem_ob_min);
                       
  //=================================================================================================================================================================
    
#if (0 != TOUCH_DEMO)
    //See if we are touched at all.
    if (0 != points_touched_mask)
    {
      //Loop through the possible touch points
      uint8_t
      mask;
      mask = 0x01;
      for (uint8_t i = 0; i < 5; i++)
      {
        if (0 != (points_touched_mask & mask))
        {
          //This code loops through all the points touched
          static uint32_t colors[5] =
          {
            EVE_ENC_COLOR_RGB(0x00, 0x00, 0xFF),
            EVE_ENC_COLOR_RGB(0x00, 0xFF, 0x00),
            EVE_ENC_COLOR_RGB(0xFF, 0x00, 0x00),
            EVE_ENC_COLOR_RGB(0xFF, 0x00, 0xFF),
            EVE_ENC_COLOR_RGB(0xFF, 0xFF, 0x00)
          };
#if (0!=DEBUG_COPROCESSOR_RESET)
          //Test code to crash coprocessor ever other time it is called --
          //for testing Reset_EVE_Coprocessor()
          DBG_STAT("Initialize_Logo_Demo() . . .");
          FWo = Initialize_Logo_Demo(FWo, &RAM_G_Unused_Start);
          DBG_STAT("  done.\n");
#endif // (0!=DEBUG_COPROCESSOR_RESET)
          FWo = EVE_Cmd_Dat_0(FWo,
                              colors[i]);
          // Make it solid
          FWo = EVE_Cmd_Dat_0(FWo,
                              EVE_ENC_COLOR_A(0xFF));
          // Draw the touch dot -- a 60px point (filled circle)
          FWo = EVE_Point(FWo,
                          x_points[i] * 16,
                          y_points[i] * 16,
                          60 * 16);
          //Tag the touch point with magenta text to show off EVE_PrintF.
          FWo = EVE_Cmd_Dat_0(FWo,
                              EVE_ENC_COLOR_RGB(0xFF, 0x00, 0xFF));
          //Move the the text out from under the user's finger
          int16_t
          xoffset;
          int16_t
          yoffset;
          if (x_points[i] < (LCD_WIDTH / 2))
          {
            xoffset = 160;
          }
          else
          {
            xoffset = -160;
          }
          if (y_points[i] < (LCD_HEIGHT / 2))
          {
            yoffset = 80;
          }
          else
          {
            yoffset = -80;
          }
          //Put the text into the display list
          FWo = EVE_PrintF(FWo,
                           x_points[i] + xoffset,
                           y_points[i] + yoffset,
                           25,         //Font
                           EVE_OPT_CENTER, //Options
                           "T[%d]@(%d,%d)",
                           i + 1,
                           x_points[i],
                           y_points[i]);

        }
        mask <<= 1;
#if (0 != MANUAL_BACKLIGHT_DEBUG)
        //Set the backlight brightness based on the first touch point.
        if (0 != (0x01 & points_touched_mask))
        {
          FWo = Set_Backlight_From_Touch(FWo, x_points[0], LCD_WIDTH);
        }
#endif // (0 != MANUAL_BACKLIGHT_DEBUG)
      }
    }
#endif // (0 != TOUCH_DEMO)

#if (0 != MARBLE_DEMO)
#if (0 != TOUCH_DEMO)
    //Only show the bouncing marble if no there is no touch
    if (0 == points_touched_mask)
    {
#endif //(0 != TOUCH_DEMO)
      FWo = Add_Marble_To_Display_List(FWo);
#if (0 != TOUCH_DEMO)
    }
#endif //(0 != TOUCH_DEMO)
#endif //(0 != MARBLE_DEMO)

#if (0 != BOUNCE_DEMO)
    //========== BOUNCY BALL $ RUBBER BAND ==========
    FWo = Add_Bounce_To_Display_List( FWo);
#endif //BOUNCE_DEMO

#if (0 != VIDEO_DEMO)
#if (0 != TOUCH_DEMO)
    FWo = Add_Video_To_Display_List(FWo, points_touched_mask, x_points, y_points);
#else // (0 != TOUCH_DEMO)
    FWo = Add_Video_To_Display_List(FWo);
#endif // (0 != TOUCH_DEMO)
#endif // (0 != VIDEO_DEMO)
#if (0 != LOGO_DEMO)
    FWo = Add_Logo_To_Display_List(FWo);
#endif // (0 != LOGO_DEMO)

#if (0 != REMOTE_BACKLIGHT_DEBUG)
    int
    byte_read;
    if (-1 != (byte_read = Serial.read()))
    {
      DBG_GEEK("Serial Data Read: %3d 0x%02X", byte_read, byte_read);
      if (byte_read <= 128)
      {
        EVE_REG_Write_8(EVE_REG_PWM_DUTY, byte_read);
        DBG_GEEK(", backlight set.");
      }
      DBG_GEEK("\n");
    }
#endif // (0 != REMOTE_BACKLIGHT_DEBUG)

#if (0 != VIDEO_DEMO)
    //Move the video to the next 30Hz frame
    FWo = Update_Video_Frame(FWo);
#endif // (0 != VIDEO_DEMO)
    //========== FINSH AND SHOW THE DISPLAY LIST ==========
    // Instruct the graphics processor to show the list
    FWo = EVE_Cmd_Dat_0(FWo, EVE_ENC_DISPLAY());
    // Make this list active
    FWo = EVE_Cmd_Dat_0(FWo, EVE_ENC_CMD_SWAP);
    // Update the ring buffer pointer so the graphics processor starts executing
    EVE_REG_Write_16(EVE_REG_CMD_WRITE, (FWo));

#if (0 != BOUNCE_DEMO)
    //========== MOVE THE BALL AND CYCLE COLOR AND TRANSPARENCY ==========
    Bounce_Ball();
#endif //(0 != BOUNCE_DEMO)

#if (0 != MARBLE_DEMO)
    //========== BOUNCE THE MARBLE AROUND ==========
    Move_Marble();
#endif //(0 != MARBLE_DEMO)
  }  // while(1)
} // loop()

//======================================================================================================================

void makeHTTPRequest() {

  Serial.println("Requesting");

  // Opening connection to server (Use 80 as port if HTTP)
  if (!client.connect(TEST_HOST, 80))
  {
    Serial.println(F("Connection failed"));
    cf_Count ++; //keep track of connection fails
    
    WiFi.end();
    delay(10);
    WiFi.begin(ssid, pass);

    return;
  }

  else
  {
    cf_Count = 0; //connection good - reset count
  }

  Serial.print("Connection Fail Count = ");
  Serial.println(cf_Count);

  // give the esp a breather
  yield();

  // Send HTTP request
  client.print(F("GET "));
  // This is the second half of a request (everything that comes after the base URL)
  client.print(SECRET_KEY);
  client.println(F(" HTTP/1.1"));

  //Headers
  client.print(F("Host: "));
  client.println(TEST_HOST);

  client.println(F("Cache-Control: no-cache"));

  if (client.println() == 0)
  {
    Serial.println(F("Failed to send request"));
    return;
  }
  //delay(100);
  // Check HTTP status
  char status[32] = {0};
  client.readBytesUntil('\r', status, sizeof(status));
  if (strcmp(status, "HTTP/1.1 200 OK") != 0)
  {
    Serial.print(F("Unexpected response: "));
    Serial.println(status);
    return;
  }

  // Skip HTTP headers
  char endOfHeaders[] = "\r\n\r\n";
  if (!client.find(endOfHeaders))
  {
    Serial.println(F("Invalid response"));
    return;
  }

  // This is probably not needed for most, but I had issues
  // with the Tindie api where sometimes there were random
  // characters coming back before the body of the response.
  // This will cause no harm to leave it in
  // peek() will look at the character, but not take it off the queue
  while (client.available() && client.peek() != '{')
  {
    char c = 0;
    client.readBytes(&c, 1);
    Serial.print(c);
    Serial.println("BAD");
  }

  DynamicJsonDocument doc(6144);

  DeserializationError error = deserializeJson(doc, client);

  if (error) {
    Serial.print(F("deserializeJson() failed: "));
    Serial.println(error.f_str());
    return;
  }
  
// The huge input: an extract from Davis Weather response
const char* credit = doc["credit"]; // "Davis Instruments Corp."
const char* credit_URL = doc["credit_URL"]; // "http://www.davisnet.com"
const char* disclaimer_url = doc["disclaimer_url"]; // "http://www.davisnet.com/about/terms.asp"
const char* copyright_url = doc["copyright_url"]; // "http://www.davisnet.com/about/terms.asp"
const char* privacy_policy_url = doc["privacy_policy_url"];

JsonObject image = doc["image"];
const char* image_url = image["url"]; // "http://www.weatherlink.com/images/Logo_Davis_reflxblu.jpg"
const char* image_title = image["title"]; // "Davis WeatherLink"
const char* image_link = image["link"]; // "http://www.weatherlink.com"

const char* suggested_pickup = doc["suggested_pickup"]; // "15 minutes after the hour"
const char* suggested_pickup_period = doc["suggested_pickup_period"]; // "60"
const char* dewpoint_c = doc["dewpoint_c"]; // "-13.3"
const char* dewpoint_f = doc["dewpoint_f"]; // "8.0"
const char* dewpoint_string = doc["dewpoint_string"]; // "8.0 F (-13.3 C)"
const char* heat_index_c = doc["heat_index_c"]; // "-9.4"
const char* heat_index_f = doc["heat_index_f"]; // "15.0"
const char* heat_index_string = doc["heat_index_string"]; // "15.0 F (-9.4 C)"
const char* location = doc["location"]; // "Pemberville, OH, United States of America"
const char* latitude = doc["latitude"]; // "41.40177"
const char* longitude = doc["longitude"]; // "-83.44719"
const char* observation_time = doc["observation_time"]; // "Last Updated on Jan 25 2022, 9:52 pm EST"
const char* observation_time_rfc822 = doc["observation_time_rfc822"]; // "Tue, 25 Jan 2022 21:52:10 ...
const char* pressure_in = doc["pressure_in"]; // "30.328"
const char* pressure_mb = doc["pressure_mb"]; // "1027.0"
const char* pressure_string = doc["pressure_string"]; // "1027.0 mb"
const char* relative_humidity = doc["relative_humidity"]; // "73"
const char* station_id = doc["station_id"]; // "larryk240"
const char* temp_c = doc["temp_c"]; // "-9.2"
const char* temp_f = doc["temp_f"]; // "15.4"
const char* temperature_string = doc["temperature_string"]; // "15.4 F (-9.2 C)"
const char* wind_degrees = doc["wind_degrees"]; // "271"
const char* wind_dir = doc["wind_dir"]; // "West"
const char* wind_kt = doc["wind_kt"]; // "5.2"
const char* wind_mph = doc["wind_mph"]; // "6.0"
const char* windchill_c = doc["windchill_c"]; // "-10.6"
const char* windchill_f = doc["windchill_f"]; // "13.0"
const char* windchill_string = doc["windchill_string"]; // "13.0 F (-10.6 C)"

JsonObject davis_current_observation = doc["davis_current_observation"];
const char* davis_current_observation_DID = davis_current_observation["DID"]; // "001D0AE07E21"
const char* davis_current_observation_station_name = davis_current_observation["station_name"];
int davis_current_observation_observation_age = davis_current_observation["observation_age"]; // 3
const char* davis_current_observation_dewpoint_day_high_f = davis_current_observation["dewpoint_day_high_f"];
const char* davis_current_observation_dewpoint_day_high_time = davis_current_observation["dewpoint_day_high_time"];
const char* davis_current_observation_dewpoint_day_low_f = davis_current_observation["dewpoint_day_low_f"];
const char* davis_current_observation_dewpoint_day_low_time = davis_current_observation["dewpoint_day_low_time"];
const char* davis_current_observation_dewpoint_month_high_f = davis_current_observation["dewpoint_month_high_f"];
const char* davis_current_observation_dewpoint_month_low_f = davis_current_observation["dewpoint_month_low_f"];
const char* davis_current_observation_dewpoint_year_high_f = davis_current_observation["dewpoint_year_high_f"];
const char* davis_current_observation_dewpoint_year_low_f = davis_current_observation["dewpoint_year_low_f"];
const char* davis_current_observation_heat_index_day_high_f = davis_current_observation["heat_index_day_high_f"];
const char* davis_current_observation_heat_index_day_high_time = davis_current_observation["heat_index_day_high_time"];
const char* davis_current_observation_heat_index_month_high_f = davis_current_observation["heat_index_month_high_f"];
const char* davis_current_observation_heat_index_year_high_f = davis_current_observation["heat_index_year_high_f"];
const char* davis_current_observation_pressure_day_high_in = davis_current_observation["pressure_day_high_in"];
const char* davis_current_observation_pressure_day_high_time = davis_current_observation["pressure_day_high_time"];
const char* davis_current_observation_pressure_day_low_in = davis_current_observation["pressure_day_low_in"];
const char* davis_current_observation_pressure_day_low_time = davis_current_observation["pressure_day_low_time"];
const char* davis_current_observation_pressure_month_high_in = davis_current_observation["pressure_month_high_in"];
const char* davis_current_observation_pressure_month_low_in = davis_current_observation["pressure_month_low_in"];
const char* davis_current_observation_pressure_tendency_string = davis_current_observation["pressure_tendency_string"];
const char* davis_current_observation_pressure_year_high_in = davis_current_observation["pressure_year_high_in"];
const char* davis_current_observation_pressure_year_low_in = davis_current_observation["pressure_year_low_in"];
const char* davis_current_observation_rain_day_in = davis_current_observation["rain_day_in"];
const char* davis_current_observation_rain_month_in = davis_current_observation["rain_month_in"];
const char* davis_current_observation_rain_rate_day_high_in_per_hr = davis_current_observation["rain_rate_day_high_in_per_hr"];
const char* davis_current_observation_rain_rate_hour_high_in_per_hr = davis_current_observation["rain_rate_hour_high_in_per_hr"];
const char* davis_current_observation_rain_rate_in_per_hr = davis_current_observation["rain_rate_in_per_hr"];
const char* davis_current_observation_rain_rate_month_high_in_per_hr = davis_current_observation["rain_rate_month_high_in_per_hr"];
const char* davis_current_observation_rain_rate_year_high_in_per_hr = davis_current_observation["rain_rate_year_high_in_per_hr"];
const char* davis_current_observation_rain_storm_in = davis_current_observation["rain_storm_in"];
const char* davis_current_observation_rain_year_in = davis_current_observation["rain_year_in"];
const char* davis_current_observation_relative_humidity_day_high = davis_current_observation["relative_humidity_day_high"];
const char* davis_current_observation_relative_humidity_day_high_time = davis_current_observation["relative_humidity_day_high_time"];
const char* davis_current_observation_relative_humidity_day_low = davis_current_observation["relative_humidity_day_low"];
const char* davis_current_observation_relative_humidity_day_low_time = davis_current_observation["relative_humidity_day_low_time"];
const char* davis_current_observation_relative_humidity_month_high = davis_current_observation["relative_humidity_month_high"];
const char* davis_current_observation_relative_humidity_month_low = davis_current_observation["relative_humidity_month_low"];
const char* davis_current_observation_relative_humidity_year_high = davis_current_observation["relative_humidity_year_high"];
const char* davis_current_observation_relative_humidity_year_low = davis_current_observation["relative_humidity_year_low"];
const char* davis_current_observation_relative_humidity_in = davis_current_observation["relative_humidity_in"];
const char* davis_current_observation_relative_humidity_in_day_high = davis_current_observation["relative_humidity_in_day_high"];
const char* davis_current_observation_relative_humidity_in_day_high_time = davis_current_observation["relative_humidity_in_day_high_time"];
const char* davis_current_observation_relative_humidity_in_day_low = davis_current_observation["relative_humidity_in_day_low"];
const char* davis_current_observation_relative_humidity_in_day_low_time = davis_current_observation["relative_humidity_in_day_low_time"];
const char* davis_current_observation_relative_humidity_in_month_high = davis_current_observation["relative_humidity_in_month_high"];
const char* davis_current_observation_relative_humidity_in_month_low = davis_current_observation["relative_humidity_in_month_low"];
const char* davis_current_observation_relative_humidity_in_year_high = davis_current_observation["relative_humidity_in_year_high"];
const char* davis_current_observation_relative_humidity_in_year_low = davis_current_observation["relative_humidity_in_year_low"];
const char* davis_current_observation_sunrise = davis_current_observation["sunrise"]; // "7:50am"
const char* davis_current_observation_sunset = davis_current_observation["sunset"]; // "5:41pm"
const char* davis_current_observation_temp_day_high_f = davis_current_observation["temp_day_high_f"];
const char* davis_current_observation_temp_day_high_time = davis_current_observation["temp_day_high_time"];
const char* davis_current_observation_temp_day_low_f = davis_current_observation["temp_day_low_f"];
const char* davis_current_observation_temp_day_low_time = davis_current_observation["temp_day_low_time"];
const char* davis_current_observation_temp_month_high_f = davis_current_observation["temp_month_high_f"];
const char* davis_current_observation_temp_month_low_f = davis_current_observation["temp_month_low_f"];
const char* davis_current_observation_temp_year_high_f = davis_current_observation["temp_year_high_f"];
const char* davis_current_observation_temp_year_low_f = davis_current_observation["temp_year_low_f"];
const char* davis_current_observation_temp_in_day_high_f = davis_current_observation["temp_in_day_high_f"];
const char* davis_current_observation_temp_in_day_high_time = davis_current_observation["temp_in_day_high_time"];
const char* davis_current_observation_temp_in_day_low_f = davis_current_observation["temp_in_day_low_f"];
const char* davis_current_observation_temp_in_day_low_time = davis_current_observation["temp_in_day_low_time"];
const char* davis_current_observation_temp_in_f = davis_current_observation["temp_in_f"]; // "49.6"
const char* davis_current_observation_temp_in_month_high_f = davis_current_observation["temp_in_month_high_f"];
const char* davis_current_observation_temp_in_month_low_f = davis_current_observation["temp_in_month_low_f"];
const char* davis_current_observation_temp_in_year_high_f = davis_current_observation["temp_in_year_high_f"];
const char* davis_current_observation_temp_in_year_low_f = davis_current_observation["temp_in_year_low_f"];
const char* davis_current_observation_wind_day_high_mph = davis_current_observation["wind_day_high_mph"];
const char* davis_current_observation_wind_day_high_time = davis_current_observation["wind_day_high_time"];
const char* davis_current_observation_wind_month_high_mph = davis_current_observation["wind_month_high_mph"];
const char* davis_current_observation_wind_ten_min_avg_mph = davis_current_observation["wind_ten_min_avg_mph"];
const char* davis_current_observation_wind_ten_min_gust_mph = davis_current_observation["wind_ten_min_gust_mph"];
const char* davis_current_observation_wind_year_high_mph = davis_current_observation["wind_year_high_mph"];
const char* davis_current_observation_windchill_day_low_f = davis_current_observation["windchill_day_low_f"];
const char* davis_current_observation_windchill_day_low_time = davis_current_observation["windchill_day_low_time"];
const char* davis_current_observation_windchill_month_low_f = davis_current_observation["windchill_month_low_f"];
const char* davis_current_observation_windchill_year_low_f = davis_current_observation["windchill_year_low_f"];

const char* time_to_generate = doc["time_to_generate"]; // "0.008630 seconds"

    if (!error) {

  // Print the result
  //serializeJsonPretty(doc, Serial);

//local
String freedom_dewpoint_f_x = doc["dewpoint_f"];
//move to global
freedom_dewpoint_f = freedom_dewpoint_f_x;

String freedom_observation_time_x = doc["observation_time"];
freedom_observation_time = freedom_observation_time_x;

String freedom_temp_f_x = doc["temp_f"];
freedom_temp_f = freedom_temp_f_x;

String freedom_wind_mph_x = doc["wind_mph"];
freedom_wind_mph = freedom_wind_mph_x;

String freedom_wind_dir_x = doc["wind_dir"];
freedom_wind_dir = freedom_wind_dir_x;

String freedom_windchill_f_x = doc["windchill_f"];
freedom_windchill_f = freedom_windchill_f_x;

String freedom_pressure_in_x = doc["pressure_in"];
freedom_pressure_in = freedom_pressure_in_x;

String freedom_relative_humidity_x = doc["relative_humidity"];
freedom_relative_humidity = freedom_relative_humidity_x;

String freedom_rain_day_in_x = davis_current_observation["rain_day_in"];
freedom_rain_day_in = freedom_rain_day_in_x;

String freedom_rain_month_in_x = davis_current_observation["rain_month_in"];
freedom_rain_month_in = freedom_rain_month_in_x;

String freedom_rain_year_in_x = davis_current_observation["rain_year_in"];
freedom_rain_year_in = freedom_rain_year_in_x;

String freedom_rain_storm_in_x = davis_current_observation["rain_storm_in"];
freedom_rain_storm_in = freedom_rain_storm_in_x;
//freedom_rain_storm_in = ("0.01");// teesting
freedom_rain_storm_flt = freedom_rain_storm_in.toFloat();
Serial.print("freedom_rain_storm_in = ");
Serial.print(freedom_rain_storm_in);
Serial.print("   ");
Serial.println(freedom_rain_storm_flt);

String freedom_pressure_tendency_string_x = davis_current_observation["pressure_tendency_string"];
freedom_pressure_tendency_string = freedom_pressure_tendency_string_x;

String freedom_sunrise_x = davis_current_observation["sunrise"];
freedom_sunrise = freedom_sunrise_x;

String freedom_sunset_x = davis_current_observation["sunset"];
freedom_sunset = freedom_sunset_x;

Serial.println(freedom_observation_time);
/////////////////////////////////// extract minutes from observation time
int delimiter, delimiter_1 ;
delimiter = freedom_observation_time.indexOf(":");
delimiter_1 = freedom_observation_time.indexOf(" ", delimiter + 1);
String freedom_ob_min = freedom_observation_time.substring(delimiter + 1, delimiter_1); //substring(beginning, end)
ob_min = freedom_ob_min.toInt();
Serial.print("ob_min = ");
Serial.println(ob_min);
Serial.print("mem_ob_min = ");
Serial.println(mem_ob_min);

  }//if != error

}// http request -------------------------------------------------------------------------------
    
